#include <stdio.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include "../driver/bttv.h"

#define VENDOR_AVER_MEDIA 0x1461
#define VENDOR_HAUPPAUGE  0x0070
#define VENDOR_FLYVIDEO   0x1851
#define VENDOR_STB        0x10B4

int fbttv;
char *bttvname = "/dev/video";
unsigned char tvee[256];

static char *chiptype[] = {
	"Reserved", "BT815", "BT817", "BT819", "BT815A", "BT817A", "BT819A",
	"BT827", "BT829", "BT848", "BT848A", "BT849A", "BT829A", "BT827A",
	"BT878", "BT879", "BT880"
};

static char *tunertype[] = {
	"NONE", "EXTERNAL", "OTHER", "Philips FI1216", "Philips FI1216MF",
	"Philips FI1236", "Philips FI1246", "Philips FI1256",
	"Philips FI1216 MK2", "Philips FI1216MF MK2", "Philips FI1236 MK2",
	"Philips FI1246 MK2", "Philips FI1256 MK2", "Temic 4032FY5",
	"Temic 4002FH5", "Temic 4062FY5", "Philips FR1216 MK2",
	"Philips FR1216MF MK2", "Philips FR1236 MK2", "Philips FR1246 MK2",
	"Philips FR1256 MK2", "Philips FM1216", "Philips FM1216MF",
	"Philips FM1236", "Philips FM1246", "Philips FM1256",
	"Temic 4036FY5", "Samsung TCPN9082D", "Samsung TCPM9092P",
	"Temic 4006FH5", "Samsung TCPN9085D", "Samsung TCPB9085P",
	"Samsung TCPL9091P", "Temic 4039FY5", "Phillips FQ1216 ME",
	"Temic 4066FY5"};

static char *sndtype[] = {
	"None", "TEA6300", "TEA6320", "TDA9850", "MSP3400C", "MSP3410D",
	"MSP3415", "MSP3430"};

static char *sndout[] = {
	"None", "Internal", "BackPanel", "Internal and BackPanel"};

static char *teletext[] = { "None", "SAA5246", "SAA5284", "Yes (Software)"};


int main()
{
	unsigned int model;
	unsigned int revision;
	unsigned int serial;
	unsigned int vendor_id;
	unsigned int id;
	int i, b1, b2, b3;

	if ((fbttv = open(bttvname, O_RDONLY)) < 0) {
		printf("Cannot open %s\n", bttvname);
		exit(1);
	}
	ioctl(fbttv, BTTV_READEE, tvee);
	id = (tvee[252] << 8) | tvee[253];
	vendor_id = (tvee[254] << 8) | tvee[255];
	printf("Card Vendor: 0x%04x, Model: 0x%04x\n", vendor_id, id);
	if (vendor_id == VENDOR_HAUPPAUGE) {
		b1 = 2 + tvee[1];
		b2 = 2 + tvee[b1 + 2];
		b3 = 2 + tvee[b1 + 2 + b2 + 1];		/* Blocklngen */
		model = (tvee[12] << 8 | tvee[11]);
		revision = (tvee[15] << 16 | tvee[14] << 8 | tvee[13]);
		serial = (tvee[b1 + 12] << 16 | tvee[b1 + 11] << 8 | tvee[b1 + 10]);
		printf("Hauppauge Model %d Rev. %c%c%c%c\n",
		       model,
		       ((revision >> 18) & 0x3f) + 32,
		       ((revision >> 12) & 0x3f) + 32,
		       ((revision >> 6) & 0x3f) + 32,
		       ((revision >> 0) & 0x3f) + 32);
		printf("Serial: %d\n", serial);
		printf("Tuner: %s\n", tunertype[tvee[9]], tvee[9]);
		printf("Audio: %s\n", sndtype[tvee[b1 + 2 + b2 + 3]]);
//              printf("Radio:\n");
//              printf("Teletext: %s\n",teletext[tvee[38]]); /* ?????? */
		printf("Decoder: %s\n", chiptype[tvee[6]]);
		printf("EEPROM:\n");
		for (i = 0; i <= 40; i++) {
			if (i == (b1 + 1) || i == (b1 + b2 + 2) 
                            || i == (b1 + b2 + b3 + 3))
				printf("\n");
			printf("%02x ", tvee[i]);
		}
		printf("\n");
	}
	return 0;
}
